#ifndef lint
static char *rcsid = "@(#)tulp:tulp-4_1_0-v3:queue.c,v 1.3 1997/09/22 22:49:20 woods Exp";
static char *copid = "@(#) Copyright (C) 1993  Christophe Wolfhugel & Herve Schauer Consultants";
#endif
/*
**  Tulp-enqueue - enqueue daemon for a UNIX Mailing Lists manager.
**
**  Copyright (C) 1994  Christophe Wolfhugel & Herve Schauer Consultants
**
**  Please read the files COPYRIGHT and AUTHORS for the extended
**  copyrights refering to this file.
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 1, or (at your option)
**  any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/cdefs.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sysexits.h>
#include "conf.h"
#ifdef FAKESYSLOG
# include "fakesyslog.h"
#else
# include <syslog.h>
#endif

/*
** Main loop.
*/
int
main(argc, argv)
   int argc;
   char *argv[];
{
   char qfile[26*2 + 2];
   char buf[BUFSIZ];
   int i, fd;

   openlog("tulp-enqueue", LOG_PID, LOG_MAIL);
   if (argc != 2) {
      syslog(LOG_WARNING, "Invalid number of arguments");
      fprintf(stderr, "%s: Invalid number of arguments (%d != 2).\n", argv[0], argc);
      exit(EX_USAGE);
   }
   if (chdir(QUEUE_DIR) == -1) {
      int oerrno = errno;

      syslog(LOG_ERR, "Can't chdir to %s: %m", QUEUE_DIR);
      fprintf(stderr, "%s: Can't chdir to %s: %s.\n", argv[0], QUEUE_DIR, strerror(oerrno));
      exit(EX_OSFILE);
   }
   umask(UMASK);			/* error check? */
   sprintf(qfile, "Tmp%ld.%d", (long) time((time_t *) NULL), getpid());
   fd = open(qfile, O_CREAT|O_WRONLY, 0600);
   if (fd == -1) {
      int oerrno = errno;

      syslog(LOG_ERR, "Could not open %s: %m", qfile);
      fprintf(stderr, "%s: Could not open %s: %s.\n", argv[0], qfile, strerror(oerrno));
      exit(EX_CANTCREAT);
   }
   if (write(fd, "X-Listserv-To: ", 15) < 15) {
      int oerrno = errno;

      syslog(LOG_ERR, "error writing %s: %m", qfile);
      fprintf(stderr, "%s: error writing %s: %s.\n", argv[0], qfile, strerror(oerrno));
      exit(EX_IOERR);
   }
   if (write(fd, argv[1], strlen(argv[1])) == -1) {
      int oerrno = errno;

      syslog(LOG_ERR, "error writing %s: %m", qfile);
      fprintf(stderr, "%s: error writing %s: %s.\n", argv[0], qfile, strerror(oerrno));
      exit(EX_IOERR);
   }
   if (write(fd, "\n", 1) < 1) {
      int oerrno = errno;

      syslog(LOG_ERR, "error writing %s: %m", qfile);
      fprintf(stderr, "%s: error writing %s: %s.\n", argv[0], qfile, strerror(oerrno));
      exit(EX_IOERR);
   }
   while ((i = read(fileno(stdin), buf, sizeof buf)) > 0) {
      if (write(fd, buf, i) < i) {
         int oerrno = errno;

         syslog(LOG_ERR, "error writing %s: %m", qfile);
         fprintf(stderr, "%s: error writing %s: %s.\n", argv[0], qfile, strerror(oerrno));
         exit(EX_IOERR);
     }
   }
   if (close(fd) == -1) {
      int oerrno = errno;

      syslog(LOG_ERR, "error closing %s: %m", qfile);
      fprintf(stderr, "%s: error closing %s: %s.\n", argv[0], qfile, strerror(oerrno));
      exit(EX_IOERR);
   }
   if (rename(qfile, qfile + 3) == -1) {
      int oerrno = errno;

      syslog(LOG_ERR, "error renaming %s to %s: %m", qfile, qfile + 3);
      fprintf(stderr, "%s: error renaming %s to %s: %s.\n", argv[0], qfile, qfile+3, strerror(oerrno));
      exit(EX_IOERR);
   }

   syslog(LOG_INFO, "enqueued message for list '%s' in '%s/%s'.", argv[1], QUEUE_DIR, qfile + 3);

   exit(0);
   /* NOTREACHED */
}
